#ifndef FDPIPE_HH
#define FDPIPE_HH

#include "FDFilter.hh"
#include "Pipe.hh"
#include "TSeries.hh"
#include <memory>

class DVector;

/**  FDPipe does time series filtering in the Fourier domain. After 
  *  constructing a FDPipe filter from the appropriate FDFilterBase, the 
  *  FDPipe can be used identically to a Pipe.
  *  @memo
  */

class FDPipe : public Pipe {
public:
  enum mode_flags {
    discrete,
    overlap,
    inmux,
    outmux
  };

public:
   using Pipe::apply;
   using Pipe::dataCheck;

  /**  Null pipe constructor.
    */
  FDPipe(void);

  /**  Filter pipe constructor.
    */
  FDPipe(const FDFilterBase& fd, mode_flags=discrete, Interval tChomp=0.0);

  /**  Copy constructor
    */
  FDPipe(const FDPipe& fp);

  /**  FDPipe destructor.
    */
  virtual ~FDPipe(void);

  /**  Check that input data is compatible with expected filter input. This
    *  method implements the Pipe API data check.
    */
  void dataCheck(const TSeries& ts) const;

  /**  Apply a filter to a time series.
    */
  TSeries apply(const TSeries& ts);

  /**  Create a copy of the FDPipie filter. This method implements the Pipe
    *  API clone method.
    */
  FDPipe* clone(void) const;

  /**  Flush out any umfiltered data.
    */
  TSeries flush(void);

  /**  Get the filter mode. The filter mode determines how data from a 
    *  time series longer the and chomp width will be combined.
    */
  mode_flags getMode(void) const;

  /**  Get the start time.
    */
  Time getStartTime(void) const;

  /**  Get the expected time for the next interval.
    */
  Time getCurrentTime(void) const;

  /**  Test whether the filter is being used.
   */
  bool inUse(void) const;

  /**  FDPipe assignment operator.
   */
  FDPipe& operator=(const FDPipe& fp);

  /**  Reset FDPipe.
   */
  void reset(void);

  /**  Set the chomp size.
   */
  void setChomp(Interval tChomp);

  /**  Set the frequency domain filter.
   */
  void setFilter(const FDFilterBase& fd);

  /**  Set the overlap mode.
   */
  void setMode(mode_flags mode);

private:
  /**  Filter a time series and place the result in mFiltered.
   */
  void filter(const TSeries& ts);

  /**  Filter a time series and add it to the results in mFiltered.
   */
  void filterAdd(const TSeries& ts);

  /**  Filter a time series and add it to the results in mFiltered.
   */
  void setMux(Interval tStep);

private:
  std::auto_ptr<FDFilterBase> mFilter;
  mode_flags mMode;
  Time     mStartTime;
  Time     mCurrentTime;
  Interval mChomp;
  Interval mTStep;  // input data time step
  Interval mDt;
  std::auto_ptr<DVector> mSinSq;
  TSeries  mData;
  TSeries  mFiltered;
};

//======================================  Inlne methods
inline Time
FDPipe::getCurrentTime(void) const {
    return mCurrentTime;
}
inline FDPipe::mode_flags 
FDPipe::getMode(void) const {
    return mMode;
}

inline Time
FDPipe::getStartTime(void) const {
    return mStartTime;
}

inline bool
FDPipe::inUse(void) const {
    return mStartTime != Time(0);
}

#endif // FDPIPE_HH
