#ifndef IIRSos_HH
#define IIRSos_HH

// this class is for initializing and manipulating 
// IIR filters implemented as second order sections
//
// A second order section is define as:
//
//         b0 + b1*z^-1 + b2*z^-2
// H(z) = -----------------------
//         1 + a1*z^-1 + a2*z^-2
//
// Edward Daw, 20th May 2001

#include "Complex.hh"

class IIRSos
{
public:

  /*
   * default constructor
   */
  IIRSos(void);

  /*
   * constructor using same conventions as CDS filter files
   */
  IIRSos(double gain, double c1, double c2, double c3, double c4);

  /*
   * constructor from second order section coefficients compatible
   * with matlab
   */
  IIRSos(double b0, double b1, double b2, 
	 double a0, double a1, double a2 );

  /*
   * constructor from complex conjugate zero pair and complex conjugate
   * pole pair
   */
  IIRSos(dComplex zero, dComplex pole, double fs);

  /*
   * constructor from two real zeros and a complex conjugate pole pair
   */
  IIRSos(double z0, double z1, dComplex poles, double fs);

  /*
   * constructor from a single real zero and a complex conjugate pole pair
   */
  IIRSos(double z, dComplex poles, double fs);

  /*
   * constructor from a complex conjugate pair of roots
   */
  IIRSos(int nzeros, dComplex roots, double fs);

  /*
   * constructor from a complex conjugate zero pair and two real poles
   */
  IIRSos(dComplex zeros, double p0, double p1, double fs);
  
  /*
   * constructor from a complex conjugate zero pair and a single real zero
   */
  IIRSos(dComplex zeros, double p, double fs);

  /*
   * constructor from two real poles and two real zeros
   */
  IIRSos( int nzeros, double z0, double z1, double p0, double p1, double fs );

  /*
   * constructor from 3 real roots, 1 or 2 zeros and 2 or 1 poles
   */
  IIRSos( int nzeros, double r0, double r1, double r2, double fs );

  /*
   * constructor from 2 real roots
   */
  IIRSos( int nzeros, double r0, double r1, double fs );

  /*
   * constructor from 1 real root
   */
  IIRSos( int nzeros, double r, double fs );

  /*
   * copy constructor
   */
  IIRSos(const IIRSos& f);

  /*
   * initialize with cds compatible coefficients
   */
  int init(double gain, double c1, double c2, double c3, double c4);

  /* 
   * initialize with matlab compatible second order section coefficients
   */
  int init(double b0, double b1, double b2, double a0, double a1, double a2);

  /*
   * initialize section with complex conjugate pole pair and
   * complex conjugate zero pair. 
   */
  int init(dComplex zeros, dComplex poles, double fs);

  /*
   * initialize from a complex conjugate zero pair and two real poles
   */
  int init(dComplex zero, double p0, double p1, double fs);

  /*
   * initialize from a complex conjugate zero pair and one real pole
   */
  int init(dComplex zero, double p, double fs);

  /*
   * initialize from a complex conjugate pair of zeros or a complex
   * conjugate pair of poles
   */
  int init(int nzeros, dComplex rootpair, double fs);

  /*
   * initialize from a single real zero and a complex conjugate pole pair
   */
  int init(double z, dComplex pole, double fs);

  /*
   *initialize from a pair of real zeros and a complex conjugate pole pair
   */
  int init(double z0, double z1, dComplex pole, double fs);

  /* 
   * initialize from a pair of real zeros and a pair of real poles
   */
  int init(int nzeros, double z0, double z1, double p0, double p1,
		   double fs);

  /*
   * initialize from three real roots, zeros placed first, maximum of
   * two poles, maximum of two zeros
   */
  int init(int nzeros, double r0, double r1, double r2, double fs);

  /*
   * initialize from two real roots, zeros placed first
   */
  int init(int nzeros, double r0, double r1, double fs);

  /*
   * initialize from a single real root, either zero or pole
   */
  int init(int nzeros, double r, double fs);

//    int init(double* zeros, unsigned int nzeros, double* poles,
//  	 unsigned int npoles, double fs);

  /*
   * dump filter coefficients, history buffers, buffer counter
   */
  int dump(std::ostream& output) const;

  /*
   * apply the filter to a single double
   */
  int apply(double data_in, double* data_out);

  /* 
   * apply filter to an array of doubles in place
   */
  int apply(double* pdata, int ndata);

  /* 
   * apply filter to an array of floats in place
   */
  int apply(float* pdata, int ndata);

  /* 
   * assignment operator
   */
  IIRSos& operator=( const IIRSos& f );

  /*
   * reset operator. Resets history buffers to zero
   */
  void reset(void);

  /*
   * Evaluate H(z^{-1})
   */
  dComplex H (const dComplex& iz) const;
  
  /*
   * get b0 coefficient
   */
  double B0() const {
     return b0; }
  /*
   * get b1 coefficient
   */
  double B1() const {
     return b1; }
  /*
   * get b0 coefficient
   */
  double B2() const {
     return b2; }
  /*
   * get a1 coefficient
   */
  double A1() const {
     return a1; }
  /*
   * get a2 coefficient
   */
  double A2() const {
     return a2; }

  /*
   * default destructor
   */
  ~IIRSos(void);

private:
  // true if the coefficients are loaded
  bool _section_loaded;
  // design sampling period
  bool _period_known;
  double _period;
  // history buffers
  double _hzminusone;
  double _hzminustwo;
  // filter coefficients
  double a1,a2,b1,b2,b0;
  // number of s plane poles, zeros represented in second order section.
  unsigned int _npoles;
  unsigned int _nzeros;
};

#endif // define IIRSos_HH






